@@ -5,6 +5,7 @@ from uuid import uuid4
 
 import grpc
 from preseem.preseem_model import BaseModelObject
+from preseem_protobuf.model.service_pb2 import SetAttachmentReq
 
 
 class FakePreseemModel:
@@ -12,6 +13,7 @@ class FakePreseemModel:
     _elements = {}  # company_uuid to {element_uuid to Element object}
     _accounts = {}  # company_uuid to {account_uuid to Account object}
     _packages = {}  # company_uuid to {package_uuid to Package object}
+    _services = {}  # company_uuid to {service_uuid to Service objects}
     _sites = {}  # company_uuid to {site_uuid to Site object}
     _elem_disc = {}  # company_uuid to {uuid to ElementDiscovery object}
 
@@ -383,3 +385,68 @@ class FakePreseemModel:
                         if getattr(site, attr) != val:
                             site.updated_at = time.time()
                         setattr(site, attr, val)
+
+    class Service(BaseModelObject):
+
+        @classmethod
+        async def Create(cls, **kwargs):
+            company_uuid = kwargs.get('company_uuid')
+            assert company_uuid
+            company = FakePreseemModel._companies[company_uuid]
+            attachment = kwargs.get('attachment')
+            obj = cls(**FakePreseemModel._new_attrs(),
+                      company_uuid=company.uuid,
+                      source=kwargs.get('source'),
+                      source_id=kwargs.get('source_id'),
+                      account_uuid=kwargs.get('account_uuid'),
+                      package_uuid=kwargs.get('package_uuid'),
+                      download_rate=kwargs.get('download_rate'),
+                      upload_rate=kwargs.get('upload_rate'),
+                      attachment=attachment,
+                      subscriber_identifier=kwargs.get('subscriber_identifier'))
+            services = FakePreseemModel._services.get(company.uuid)
+            if services is None:
+                services = FakePreseemModel._services[company_uuid] = {}
+            services[obj.uuid] = copy.deepcopy(obj)
+            return copy.deepcopy(obj)
+
+        @classmethod
+        async def List(cls, **kwargs):
+            company_uuid = kwargs.get('company_uuid')
+            assert company_uuid
+            # the real impl returns empty list if the company doesn't exist
+            return [
+                copy.deepcopy(x)
+                for x in (FakePreseemModel._services.get(company_uuid) or {}).values()
+            ]
+
+        @classmethod
+        async def Delete(cls, **kwargs):
+            company_uuid = kwargs.get('company_uuid')
+            uuid = kwargs.get('uuid')
+            services = FakePreseemModel._services[company_uuid]
+            if services:
+                service = services.get(uuid)
+                if service:
+                    del services[uuid]
+
+        @classmethod
+        async def Update(cls, **kwargs):
+            company_uuid = kwargs.get('company_uuid')
+            uuid = kwargs.get('uuid')
+            services = FakePreseemModel._services[company_uuid]
+            if services:
+                service = services.get(uuid)
+                for attr, val in kwargs.items():
+                    if attr == 'attachment':
+                        if val:
+                            assert isinstance(val, SetAttachmentReq)
+                            val = val.attachment
+                    if attr == 'company_uuid' or attr == 'uuid':
+                        continue
+                    elif attr == 'inactive' and val is True:
+                        FakePreseemModel._set_inactive(service)
+                    else:
+                        if getattr(service, attr) != val:
+                            service.updated_at = time.time()
+                        setattr(service, attr, val)
